
#include "Msg.h"


/* ---------------------------------------------
 * Copy the content of one msg to another.
 * 
 * Parameters :
 * 	struct Msg *dst : destiny msg
 * 	struct Msg *src : source msg
 * 	
 * ---------------------------------------------
 */ 
extern void msgcpy (struct Msg *dst, struct Msg *src) {
    strcpy (dst->s, src->s);
}

/* ---------------------------------------------
 * return the length of a msg 
 * 
 * Parameters :
 * 	struct Msg *msg : msg
 * 	
 * ---------------------------------------------
 */ 
extern int msglen (struct Msg *msg) {
    return strlen (msg->s);
}

/* ---------------------------------------------
 * Copy the content of the array to the msg.
 * 
 * Parameters :
 * 	struct Msg *dst : destiny msg
 * 	char *src : source array
 * 	
 * Return :
 * 	int : > 0 if OK
 * 	      < 0 if not
 * ---------------------------------------------
 */ 
extern int msgset (struct Msg *dst, char *src) {
    int size;
    
    size = strlen(src);
    if (size>MAX)
       return -1;
    
    msgclean (dst,'\0');
    strncpy (dst->s,src,size);
    dst->s[MAX]='\0';
 
    return 1;    
}

/* ---------------------------------------------
 * Clean up the msg.
 * 
 * Parameters :
 * 	struct Masg *msg : msg
 * 	char c : clean up character
 * ---------------------------------------------
 */
extern void msgclean (struct Msg *msg, char c) {
    int i;
    for (i=0;i<MAX;i++)
        msg->s[i]=c;
}



/* ---------------------------------------------
 * Copy to the char array the content of the 
 * message 
 * 
 * Parameters :
 * 	struct Msg *msg : source msg
 * 	char *dst : char array
 * 	int len : dst array length
 * Return :
 * 	int : > 0 if OK
 * 	      < 0 if not
 * ---------------------------------------------
 */ 
extern int msgget (struct Msg *msg, char *dst, int len) {
    int size;

    size = strlen(msg->s);
    if (len<size)
        return -1;
      
    strcpy (dst,msg->s);    

    return 1;    
}


/* ---------------------------------------------
 * Copy the content of the msg to a new string. 
 * 
 * Parameters :
 * 	struct Msg *msg : source msg
 * Return :
 * 	AwString : string with the msg content
 * 		(or NULL, if it fails)
 * ---------------------------------------------
 */
extern AwString msgtoString (struct Msg *msg) {
    int size;	
    AwString str;
    
    size = strlen (msg->s);
    str = AwString_new (size);
    AwString_set (str,msg->s);

    return str;
}

