/* -----------------------------------------------------------------
 ** Header file for a producer / customer algorithm implementation
 **
 ** Author : Manuele Kirsch Pinheiro 
 ** Project CEMT - II/UFRGS - Opera Group/INRIA
 ** Finance by CNPq(Brazil)/INRIA(France) Cooperation
 **  
 ** Date : 18 / sep / 2001
 **            	  	
 ** $Id: prodcust.h,v 1.3 2001/11/08 15:02:03 kirschpi Exp kirschpi $
 ** $Date: 2001/11/08 15:02:03 $
 ** $Author: kirschpi $
 ** $Revision: 1.3 $
 ** $Log: prodcust.h,v $
 ** Revision 1.3  2001/11/08 15:02:03  kirschpi
 ** The shared memory struct is now declared in the header file.
 **
 ** Revision 1.2  2001/11/06 10:03:38  kirschpi
 ** Some functions that weren't present in this header
 ** have been added.
 **
 ** Revision 1.1  2001/11/05 13:44:38  kirschpi
 ** Initial revision
 **
 ** 
 * ------------------------------------------------------------------ */
 
  
#ifndef PRODCUST_H
#define PRODCUST_H

#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>

#include <sys/types.h>	/* header files needed for IPC Semaphors and Shared memory */
#include <sys/wait.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/shm.h>

#include "Msg.h"	/* file with the message description */

/* -------------------------------------------------------------------------- */

/* Struct needede for IPC semaphors */
#if defined(__GNU_LIBRARY__) && !defined(_SEM_SEMUN_UNDEFINED)
/* union semun is defined by including <sys/sem.h> */
#else
/* according to X/OPEN we have to define it ourselves */
       union semun {
               int val;                    /* value for SETVAL */
               struct semid_ds *buf;       /* buffer for IPC_STAT, IPC_SET */
               unsigned short int *array;  /* array for GETALL, SETALL */
               struct seminfo *__buf;      /* buffer for IPC_INFO */
       };
#endif

/* -------------------------------------------------------------------------- */

/* extern message for shared memory */

/* This Should be declared at Msg.* file, created by the user */
/* extern struct Msg; */
/* extern void msgcpy (struct Msg *dst, struct Msg *src);*/
#ifndef MSGMAX
#define MSGMAX	25
#endif


/* struct for shared memory */
struct shBuffer {
	struct Msg buffer [MSGMAX];
	int last;
	int size;
    	int stop;
       	int next;
};



/* -------------------------------------------------------------------------- */

/* definitions for the semaphors */
#define LOCK  	-1	/* to lock */
#define UNLOCK	1	/* to unlock */
#define DATA  	0	/* semaphor for data access */
#define EMPTY	1	/* semaphor for empty data area */
#define FULL 	2	/* semaphor for full data area */
#define NSEMA 	3	/* number of semaphors used in semaphor set */

#ifndef FALSE
#define FALSE	0
#endif


/* -------------------------------------------------------------------------- */

/* global variables */
static int pc_semid = -1;		/* identifier to the semaphors */
static int pc_shmid = -1;		/* identifier to shared memory */
char *shmp; 				/* Pointer to link the data area to the shared memory */
struct shBuffer *pc_shdata;		/* data area in the shared memory */



/* -------------------------------------------------------------------------- */

/* Operations with semaphores. Usually in the file pc_shsem.c */
extern void create_sem (void);		/* create the semaphors set */
extern void free_sem (void);		/* free the semaphors set */
extern void lock_unlock (int op, int sema, FILE *fout);		/* lock or unlock semaphors in the set */



/* -------------------------------------------------------------------------- */

/* Operations with the shared memory. Usually in the file pc_shsem.c */
extern void create_shmem ();		/* create the shared memory area */
extern void deattach_shmem (void); 	/* deattach the data from the shared memory */
extern void free_shmem (void);		/* free the shared memory */ 



/* -------------------------------------------------------------------------- */

/* Interface operations.  Usually in the file prodcust.c */
extern int Start (void);		/* start the activities */
extern void Free (void);		/* free shared memory */
extern void Finalize (void);		/* finalize all operations */
extern int PutSharedData (struct Msg *data); 		/* insere new data _ producer */
extern int GetSharedData (struct Msg *data);		/* read data _ custumer */ 
extern int CompactSharedData ();        		/* compact shared memory */
extern int GetNotLockSharedMemory (struct Msg *data);	/* read data, but do not lock 
							   itself if the buffer is empty */
extern int PutNotLockSharedData (struct Msg *data); 	/* put data, but do not lock 
							   itself if the buffer is full */
extern int ReadStop (void); 		/* return the value of the stop element in shBuffer struct */
extern void SetStop (int value);	/* set the value of the stop element in shBuffer struct */

#endif /* PRODCUST_H */
