/* --------------------------------------------------------
 **  Function for shared memory and semaphors manipulation 
 **  in producer / customer  
 **
 ** Author : Manuele Kirsch Pinheiro
 ** Project CEMT - II/UFRGS - Opera Group/INRIA
 ** Finance by CNPq(Brazil)/INRIA(France) Cooperation
 **   
 ** Date : 18 / sep / 2001
 **
 ** $Id: pc_shsem.c,v 1.2 2001/11/06 10:07:18 kirschpi Exp kirschpi $
 ** $Date: 2001/11/06 10:07:18 $
 ** $Author: kirschpi $
 ** $Revision: 1.2 $
 ** $Log: pc_shsem.c,v $
 ** Revision 1.2  2001/11/06 10:07:18  kirschpi
 ** Some unused variable has been removed.
 **
 ** Revision 1.1  2001/11/05 13:53:28  kirschpi
 ** Initial revision
 **
 ** 
 ** --------------------------------------------------------
 */
 
#include "prodcust.h"

/* function to create and set the semaphors set */
void create_sem (void) {
  int code = -1;
  static ushort i_val[NSEMA];
  union semun semarg;     /* semctl() arg */
  
	pc_semid = semget(IPC_PRIVATE,NSEMA,0600);	/* create the set  */
    	if ( pc_semid == -1 ) {
        	perror("semget()");
        	exit(1);
    	}

	printf ("Semid : %d \n",pc_semid);
	
	i_val[DATA] = UNLOCK;
	i_val[EMPTY] = 0;
	i_val[FULL] = 0;
	semarg.array = i_val;			
	code = semctl(pc_semid,0,SETALL,semarg);	/* set the semaphors */
    	if ( code == -1 ) {
        	perror("semctl(SETALL)");
        	exit(1);
    	}
}


/* free the semaphor set */
void free_sem (void) {
   int code;
   union semun semarg;     /* semctl() arg */
	
	code = semctl(pc_semid,0,IPC_RMID,semarg);	/* IPC_RMID to free */
	if ( code == -1 ) {
        	perror("semctl(IPC_RMID)");
        	exit(1);
    	}

}



/* functon to lock / unlock a semaphor in the set 
 * op = LOCK	lock the semaphor indicated by sema
 * op = UNLOCK 	unlock the semaphor indicated by sema
 */
void lock_unlock (int op, int sema, FILE *fout) {
   	struct sembuf sops[1];
	int code = FALSE;
	
   	sops[0].sem_num = sema;		/* qual semaforo no conjunto : 0 -  primeiro */
   	sops[0].sem_op = op;		/* operacao : -1 - decrementar/lock */
					/*  	      +1 - incrementar/unlock */
	sops[0].sem_flg = 0;    	/* No flags */

	if (fout !=NULL)
		fprintf (fout,"%s\n",(op==LOCK)?"LOCK":"UNLOCK");
	
	code = semop (pc_semid,sops,1);	/* realiza a operacao */
	if (code == -1) {		/* 1 final eh o numero de posicoes do vetor sops */
        	perror("semop()");      /* erro na operacao */
        	exit(13);
	}
}




/* create and attach the shared memory */
void create_shmem () {
  /* int code; */
 
	pc_shmid = shmget(IPC_PRIVATE,sizeof *pc_shdata,0666);	/* cria a memoria compartilhada */
        if ( pc_shmid == -1 ) {
            perror("shmget()");
            exit (13);
        }

	printf ("Shmid : %d \n",pc_shmid);

    	shmp = (char *) shmat(pc_shmid,0,0);	/* liga a memoria compartilhada a um */
    	if ( shmp == (char *)(-1) ) {		/* ponteiro para manipulacao */
        	perror("shmat()");
        	exit(13);
    	}
 	pc_shdata = (struct shBuffer *) shmp;

	printf ("Conected!\n");
}


/* deattach the pointer to shared memory */
void deattach_shmem (void) {
    int code=0;
    
	code = shmdt (pc_shdata);		/* desatacha o ponteiro a memoria compartilhada */
    	if ( code == -1 ) {
    	    perror("shmdt");
    	    exit(1);
    	}
}


/* free the shared memory area */
void free_shmem (void) {
    int code=0;
    struct shmid_ds buf;

	code = shmctl (pc_shmid, IPC_RMID,&buf);	/* remove a memoria compartilhada */
	if ( code == -1 ) {
    	    perror("shmdt");
    	    exit(1);
    	}
}



