/* --------------------------------------------------------
 ** Interface functions for a producer/customer algorithm
 ** implementation.
 **
 ** Author : Manuele Kirsch Pinheiro
 ** Project CEMT - II/UFRGS - Opera Group/INRIA
 ** Finance by CNPq(Brazil)/INRIA(France) Cooperation
 **   
 ** Date :	18 / sep / 2001
 ** 
 ** $Id: prodcust.c,v 1.2 2001/11/06 10:07:18 kirschpi Exp kirschpi $
 ** $Date: 2001/11/06 10:07:18 $
 ** $Author: kirschpi $
 ** $Revision: 1.2 $
 ** $Log: prodcust.c,v $
 ** Revision 1.2  2001/11/06 10:07:18  kirschpi
 ** Some unused variable has been removed.
 **
 ** Revision 1.1  2001/11/05 13:53:28  kirschpi
 ** Initial revision
 **
 ** 
 ** --------------------------------------------------------
 */

#include "prodcust.h"

/* Start the semaphor and shared memory start */
extern int Start (void) {
	/* criando a area de memoria compartilhada */
	create_shmem();
	
	/* criando os semaforos */ 
	create_sem ();

	printf ("Setting variables \n");		
	pc_shdata->last=0;
	pc_shdata->size=MSGMAX;
	pc_shdata->stop=FALSE;
	pc_shdata->next=0;

	return !FALSE;
}	


/* free the pointer to shared memory */
extern void Free (void) { 
	printf ("Deattaching shared memory \n");
	deattach_shmem ();
}


/* finalize shared memory and semaphors */
extern void Finalize (void) {
	printf ("Free _ shared memory \n");	
	free_shmem();	
	printf ("Free _ semaphors set \n");	
	free_sem();
}


/* Introduce new data in the buffer. 
 * Return -1 if failed, 0 if don't */
extern int PutSharedData (struct Msg *data) {
   int flag = 0, stop = 0;
   int ret = 0;
   struct Msg m;
   
   	//printf ("Putting data\n");
	
	lock_unlock (LOCK,DATA,NULL);
	
	stop = pc_shdata->stop;
	
	/* test if the buffer is full */
	if (pc_shdata->last > 0 && pc_shdata->last >= pc_shdata->size) {
		if (CompactSharedData()<0) {
			//printf ("Put Failed\n");
			lock_unlock (UNLOCK,DATA,NULL); /* free to the others */
			
			if (!stop) {			
				lock_unlock (UNLOCK,EMPTY,NULL); /* wake up customer */
				lock_unlock (LOCK,FULL,NULL);
				msgcpy (&m,data);
				ret = PutNotLockSharedData (&m);
				flag = -1;
			}
			else	
				return -1;	/* stop is true */
		}
	}
	
	if (flag>=0) {
		/* insere the message */
		msgcpy (&pc_shdata->buffer[pc_shdata->last++],data);
		//printf ("After Put -> Next : %d Last : %d \n",pc_shdata->next,pc_shdata->last);
		lock_unlock (UNLOCK,DATA,NULL);
	
		/* now it isn't empty */
		lock_unlock (UNLOCK,EMPTY,NULL);

		ret = 0;
	}
	
	return ret;
}


/* Get next data from the buffer (if the stop flag isn't set).
 * Return -1 if failed, 0 if don't 
 * Failed if the stop flag is set and/or there is no more data to read
 * */
extern int GetSharedData (struct Msg *data) {
   /*int flag = 1;*/
   int stop = 0, ret = 0;
   struct Msg m;
   
   	//printf ("Getting data\n");

	/* lock DATA semaphor */
	lock_unlock (LOCK,DATA,NULL);
	
	//printf ("Get -> Next : %d Last : %d \n",pc_shdata->next,pc_shdata->last);
	
	stop = pc_shdata->stop;
	
	/*test if the buffer is empty */
	if (pc_shdata->last <= 0 || pc_shdata->next >= pc_shdata->last) {
		//printf ("Get failed\n");
		lock_unlock (UNLOCK,DATA,NULL); /* unlock for other process */	
		
		if (!stop) {/* wait for data */
			lock_unlock (LOCK,EMPTY,NULL);
			ret = GetNotLockSharedMemory (&m);
			if (ret>=0)
				msgcpy (data,&m);
		}
		else
			return -1; /* stop is true */		
	}
	else {
	
		msgcpy (data, &pc_shdata->buffer[pc_shdata->next++]);
		ret = 0;
		
		/* unlock DATA semaphr */	
		lock_unlock (UNLOCK,DATA,NULL);
		
		/* now it's not full anymore */
		lock_unlock (UNLOCK,FULL,NULL);
	}
		
	
	return ret;
}


/* Compact the shared buffer.
 * Return -1 if failed, 0 if don't */
int CompactSharedData () {
    int flag = 0;
    int i,j = 0;
    
    	/* test if it is possible to compact the buffer */
	printf ("Compacting Next : %d Last : %d - ",pc_shdata->next,pc_shdata->last);
	if (pc_shdata->last >= pc_shdata->size && pc_shdata->next > 0) {
		for (i=0,j=pc_shdata->next; j<pc_shdata->size; i++,j++) {
			msgcpy (&pc_shdata->buffer[i],
				&pc_shdata->buffer[j]);
		}
		
		pc_shdata->next = 0;
		pc_shdata->last = i;
	}
	else  {
		flag = -1;	
	}
	printf ("Success : %d Next : %d Last : %d\n",flag,pc_shdata->next,pc_shdata->last);
	return flag;
}

/* Read the buffer until it is empty, but it doesn't "lock" 
 * the process when the buffer is really empty */
extern int GetNotLockSharedMemory (struct Msg *data) {
     int flag=1;
     
	//printf ("Getting data _ Not locking if empty \n");

	/*test if the buffer is empty */
	lock_unlock (LOCK,DATA,NULL);
	//printf ("Getting NL - Next : %d Last : %d \n",pc_shdata->next,pc_shdata->last );
	if (pc_shdata->last <= 0 || pc_shdata->next >= pc_shdata->last)
		flag = 0;
	else {
		/* copy the data from the buffer */
		msgcpy (data, &pc_shdata->buffer[pc_shdata->next++]);
	}	
	lock_unlock (UNLOCK,DATA,NULL);

	
	if (!flag) { /* it is empty  - do not lock anything */
		//printf ("Get failed\n");
		return -1; 
	}
	
	/* now it's not full anymore */
	lock_unlock (UNLOCK,FULL,NULL);
	
	return 0;
}


/* Insere new data in the buffer, without lock if the buffer is full */
extern int PutNotLockSharedData (struct Msg *data) {
   int flag = 0, stop = 0;
   int ret = 0;
   /* struct Msg m; */
   
   	//printf ("Putting data\n");
	
	lock_unlock (LOCK,DATA,NULL);
	
	stop = pc_shdata->stop;
	
	/* test if the buffer is full */
	if (pc_shdata->last > 0 && pc_shdata->last >= pc_shdata->size) {
		if (CompactSharedData()<0) {
			lock_unlock (UNLOCK,DATA,NULL);
			//printf ("Put Failed\n");
			ret = -1;
			flag = -1;
		}
		else flag = 0;
	}
	
	if (flag>=0) {
		/* insere the message */
		msgcpy (&pc_shdata->buffer[pc_shdata->last++],data);
		//printf ("After Put -> Next : %d Last : %d \n",pc_shdata->next,pc_shdata->last);
		lock_unlock (UNLOCK,DATA,NULL);
	
		/* now it isn't empty */
		lock_unlock (UNLOCK,EMPTY,NULL);

		ret = 0;
	}
	
	return ret;
}





/* Read the value of "stop" flag in the shBuffer struct */
extern int ReadStop (void) {
	int flag=0;
		
	flag = pc_shdata->stop;

	return flag;
}

/* Set the value of "stop" flag in the shBuffer struct */
extern void SetStop (int value) {

	printf ("Setting STOP flag \n");
	lock_unlock (LOCK,DATA,NULL);
	pc_shdata->stop = value;
	lock_unlock (UNLOCK,DATA,NULL);

	if (value) { /* to prevent deadlock - unlock all with stop signal */
		lock_unlock (UNLOCK,FULL,NULL);
		lock_unlock (UNLOCK,EMPTY,NULL);
	}


}
