#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <errno.h>

#include "awparser.h"
#include "prodcust.h"

/*struct shBuffer *pc_shdata; */     /* data area in the shared memory */

size_t readBuffer (char *buf, size_t len, FILE *file);
void showTree (AwNode *n,FILE *output);
void treeInfo (AwTree *tree,FILE *output);
void xmlprodutor (int loops);
void xmlconsumidor (FILE *output);
void cleanUp (char *buf,char c,int len); 

void cleanUp (char *buf,char c,int len) {
 int i;

 for (i=0;i<len;i++) 
	buf[i]=c;
 
}

size_t readBuffer (char *buf, size_t len, FILE *file)
{
    size_t readed;
    readed = fread(buf, 1, len, file);
    if (buf[readed-1]=='\n')
	    buf[readed-1]='\0';
    return readed;
}



void showTree (AwNode *n,FILE *output) {
    AwNode *child;
    AwPair *atr;
    int i;


    for (i = 0; i < AwNode_getLevel(n); i++)
            fputc ('\t',output);
    fprintf (output,"%c %s\n",(AwNode_getType(n)==XML_ELEMENT)?'+':' ',AwNode_getInfo(n));

    AwNode_resetAttributes(n);
    atr = AwNode_nextAttribute(n);
    while (atr!=NULL) {
        for (i = 0; i < AwNode_getLevel(n); i++)
            fputc ('\t',output);
        fprintf (output,"%s=\'%s\'\n",AwPair_getName(atr),AwPair_getValue(atr));
        atr = AwNode_nextAttribute(n);
    }

    AwNode_resetChildren (n);
    child = AwNode_nextChild(n);
    while (child!=NULL) {
        showTree(child,output);
        child = AwNode_nextChild(n);
    }

    if (AwNode_getType(n)==XML_ELEMENT) {
        for (i = 0; i < AwNode_getLevel(n); i++)
            fputc ('\t',output);
        fprintf (output,"- %s\n",AwNode_getInfo(n));
    }

}


void treeInfo (AwTree *tree,FILE *output) {

   fprintf (output,"\troot %s maxdepth %d\n",(AwTree_getRoot(tree)==NULL)?"NULL": \
                                    AwNode_getInfo(AwTree_getRoot(tree)), \
                                    AwTree_getMaxDepth(tree));

   fprintf (output,"\tlast %s depth %d\n",(AwTree_getLast(tree)==NULL)?"NULL": \
                                    AwNode_getInfo(AwTree_getLast(tree)), \
                                    AwTree_getDepth(tree));

   showTree(AwTree_getRoot(tree),output);

}



/* cria as mesgs */
void xmlprodutor (int loops) {
   char buf[MAX];	
   struct Msg m;
   int i,r;
   size_t len;
   AwTree *tree = NULL;
   AwString mesg,mesg2;

   for (i=0;i<loops;i++) {
	printf ("\ncleaning buffer\n");   
	cleanUp(buf,'\0',MAX);
     	
   	len = readBuffer (buf,sizeof(m.s),stdin);
        if (len>0) { 
       	   printf ("setting string to %s (%d)\n",buf,len);
	   mesg = AwString_new (len);
	   AwString_set (mesg,buf);	   
	   printf ("\nmesg %s\n",mesg);
		   
	   printf ("Parsing.. umount message \n");
	   tree = AwParser_umountMessage(mesg);

	   printf ("Tree's info : \n");
	   treeInfo(tree,stdout);
	   
	   printf ("Building the message protocol \n");
	   mesg2 = AwParser_mountMessage(tree);
	   printf ("%s\n",(char *)mesg2);
   
	   msgset (&m,AwString_get(mesg2));
   
	   /* colocando na memoria compartilhada */
	   r = PutSharedData (&m);
	   if (r<0){
	       printf ("PUT FAILED\n");
	   } else
		
           cleanUp(buf,'\0',MAX);		   
	   msgget (&m,buf,MAX-1);	   
	   printf ("PUT = r : %d Msg : %s (%d)\n",r,buf,msglen(&m));
   
	   AwString_delete(mesg);
	   AwString_delete(mesg2);
	   AwTree_delete(tree);
	}   
    }	   
   
    SetStop (!FALSE);
}



/* mostra as mesgs */
void xmlconsumidor (FILE *output) {
   int flag = FALSE;
   AwTree *tree = NULL;
   AwString mesg;
   struct Msg m;
   int r=0;
   char buf[MAX];

   while (!flag){
		cleanUp(buf,'\0',MAX);
		msgclean(&m,'\0');
	   
                r = GetSharedData (&m);
                flag = ReadStop();
		msgget (&m,buf,MAX-1);
                if (r<0)
                        fprintf (output,"GET FAILED - ");
                else 
                        fprintf (output,"GET = r : %d Msg : %s (%d)- ",r,buf,msglen(&m));
			
                fprintf (output,"Stop? %d \n",flag);

	   	fprintf (output,"Parsing.. umount message \n");
	   	mesg = msgtoString(&m);
		tree = AwParser_umountMessage(mesg);

	   	fprintf (output,"Tree's info : \n");
	   	treeInfo(tree,output);

		AwString_delete(mesg);
		AwTree_delete(tree);

   }

   do {
		cleanUp(buf,'\0',MAX);
		msgclean(&m,'\0');
		msgget (&m,buf,MAX-1);
		
                r = GetNotLockSharedMemory (&m);
                fprintf (output,"GET = r : %d Msg : %s \n ",r,(r<0)?"Ignored":buf);
   } while (r>=0);

}



int main (int argc, char **argv) {
  pid_t pid;
  FILE *tty;
  int loops;

  	/* pegando numero de interacoes */
  	loops = atoi (argv[1]);
		
  	/*abrindo tty para o filho */
	if ( (tty = fopen (argv[2],"r+")) == NULL) {
		fprintf (stderr,"\nNao foi possivel abrir TTY de Entrada.\n");
		fprintf (stderr,"Error :%s",strerror(errno));
		exit (0);
	}
	else printf ("TTY %s aberto \n",argv[2]);
  
        printf ("Starting \n ");	
        Start();

        /*criando o segundo processo */
        pid = fork();

        if ( pid == -1 ) { /*Fork failed to create a process */
                fprintf(stderr,"%s: Failed to fork()\n",strerror(errno));
                exit(13);

        } else if ( pid == 0 ) { /*This is the child process running */
                fprintf(stdin,"PID %ld: Child started, parent is %ld.\n",
                        (long)getpid(),         /* Our PID */
                        (long)getppid());       /* Parent PID */
               // srand ((unsigned int)getpid()+(unsigned int)getppid());
		freopen (argv[2],"a+",stderr);
                xmlconsumidor(tty);
                                                /* memoria compartilhada deve ser 
                                                 * liberada em todos os processos */

        } else  { /* This is the parent process running */
                printf("PID %ld: Started child PID %ld.\n",
                        (long)getpid(),     /* Our PID */
                        (long)pid);         /* Child's PID */
               // srand ((unsigned int)getpid());
                xmlprodutor (loops);
                Free();                 /* memoria compartilhada deve ser 
                                         * liberada em todos os processos */
                exit (0);
        }



        Free();
        Finalize();

	return 0;
}
