#include "expat.h"
#include "awtree.h"


#define NUM_OF_NODES 3
#define XML_ELEMENT 1
#define TEXT_ELEMENT 2


XML_Parser parser;


static void
textElement (void *userData,const XML_Char *s,int len) 
{
   int i;
   AwTree *tree = userData;
   AwNode *node;
   AwString info;
   char *data;

   data = (char *) AW_CALLOC (len+1,sizeof (char));
   strncpy (data,s,len);
   data[len]='\0';
  
   if (strcmp(data,"\n")==0 )		/* ignora os enter vazios */
	   return;
		 
   info = AwString_new (len);		/* criando o novo no */
   AwString_set (info,data); 
   node = AwNode_new(NUM_OF_NODES,NUM_OF_NODES,len); 
   AwNode_setInfo (node,info); 
   AwNode_setType (node,TEXT_ELEMENT);
   AwString_delete (info);
   
   //printf ("putting node %s \n",AwNode_getInfo(node)); 
   AwTree_putNewNode (tree,node);	/* coloca na arvore */

					
   for (i = 0; i < AwNode_getLevel(node); i++) 	   /* identa a saida e mostra os dados*/
    		putchar('\t');
   puts (AwNode_getInfo(node));	
}


static void
startElement(void *userData, const char *name, const char **attr)
{
  int i;
  AwTree *tree = userData;
  AwNode *node;
  AwString info,val; 
  AwPair *pair;
	  
  info = AwString_new (strlen(name));		/* criando o novo no */
  AwString_set (info,(char *)name); 
  node = AwNode_new(NUM_OF_NODES,NUM_OF_NODES,strlen (name)); 
  AwNode_setInfo (node,info); 
  AwNode_setType (node,XML_ELEMENT);
  AwString_delete (info);
  
  for (i = 0; attr[i]; i += 2) {	/* coloca os atributos */	
	  info = AwString_new (strlen(attr[i]));
	  val = AwString_new (strlen(attr[i+1]));
	  AwString_set (info,(char *)attr[i]);
	  AwString_set (val,(char *)attr[i+1]);
	  
	  pair = AwPair_new (AwString_len(info),AwString_len(val));
	  AwPair_setName(pair,info);
	  AwPair_setValue(pair,val);
	  
	  AwString_delete (info);
	  AwString_delete (val);

	  AwNode_putAttribute (node,pair);
  }
 
  //printf ("putting node %s\n",AwNode_getInfo(node)); 
  AwTree_putNewNode (tree,node);	/* coloca na arvore */  
  
  for (i = 0; i < AwNode_getLevel(node); i++) {	/* mostra */
    putchar('\t');
    putchar('.');
  }

  printf ("%s\n",AwNode_getInfo(node));
  
  AwNode_resetAttributes(node);
  pair = AwNode_nextAttribute(node);
  while (pair!=NULL) {
    for (i = 0; i < AwNode_getLevel(node) + 1 ; i++)	/* mostra atributos */
       putchar('\t');    
    printf ("%s = %s \n",AwPair_getName(pair),AwPair_getValue(pair));
    pair = AwNode_nextAttribute(node);
  }
  
}


static void
endElement(void *userData, const char *name)
{
  AwTree *tree = userData;
  AwNode *last = NULL;
  
  last = AwTree_getLast(tree);
  if (AwNode_getType(last)==TEXT_ELEMENT)	/* with text element, we need to get up twice */
	  AwTree_up(tree);               	/* to reach the last xml element */
  
  AwTree_up(tree);
}



size_t readBuffer (char *buf, size_t len, FILE *file) 
{
	size_t readed;
	readed = fread(buf, 1, len, file);
	return readed;
}


void freenode (AwNode *n) {
	AwNode *child=NULL;
	AwPair *atr=NULL;
	int i;
	
	//printf ("going...\n");
	
	for (i=0;i<AwNode_getLevel(n);i++) {
		putchar('\t');
		putchar('.');
	}
	printf ("%s\n",AwNode_getInfo(n));

	
  	AwNode_resetAttributes(n);
	atr = AwNode_nextAttribute(n);
  	while (atr!=NULL) {
    		for (i = 0; i < AwNode_getLevel(n) + 1 ; i++)	/* mostra atributos */
       			putchar('\t');    
    		printf ("%s = %s \n",AwPair_getName(atr),AwPair_getValue(atr));
		AwPair_delete (atr);
    		atr = AwNode_nextAttribute(n);
  	}

	AwNode_resetChildren (n);
	child = AwNode_nextChild(n);
	while (child!=NULL) {
		freenode(child);
		child = AwNode_nextChild(n);
	}

	if (AwNode_getType(n)==XML_ELEMENT) {
		for (i=0;i<AwNode_getLevel(n);i++)
			putchar('\t');
		puts(AwNode_getInfo(n));
	}	
  

	AwNode_delete(n);	
}

int
main(int argc, char *argv[])
{
  char buf[BUFSIZ];
  int done;
  //int depth = 0;
  //int flag = -1;
  AwTree *tree = NULL;
  
  tree = AwTree_new ();
  
  parser = XML_ParserCreate(NULL);
  XML_SetUserData(parser, tree);
  XML_SetElementHandler(parser, startElement, endElement);
  XML_SetCharacterDataHandler(parser,textElement);

  do {
    size_t len = readBuffer (buf,sizeof(buf),stdin);
    done = len < sizeof(buf);
    
    if (!XML_Parse(parser, buf, len, 0 )) { 
      fprintf(stderr,
	      "%s at line %d\n",
	      XML_ErrorString(XML_GetErrorCode(parser)),
	      XML_GetCurrentLineNumber(parser));
      return 1;
    }
    
    /* printf ("Readed %d done %d flag %d\n",len,done,flag);*/
    
  } while (!done);
  
  printf ("Tree's info : \n");
  printf ("\troot %s maxdepth %d\n",(AwTree_getRoot(tree)==NULL)?"NULL": \
					AwNode_getInfo(AwTree_getRoot(tree)),
					AwTree_getMaxDepth(tree));

  printf ("\tlast %s depth %d\n",(AwTree_getLast(tree)==NULL)?"NULL": \
					AwNode_getInfo(AwTree_getLast(tree)),
					AwTree_getDepth(tree));

	
  /*libera*/
  printf ("freeing \n");
  freenode(AwTree_getRoot(tree));
  AwTree_delete(tree);
  XML_ParserFree(parser); 
  
  return 0;
}



