#include "expat.h"
#include "awtree.h"


#define NUM_OF_NODES 3
#define XML_ELEMENT 1
#define TEXT_ELEMENT 2

//XML_Parser parser;


AwNode *makeNode (char *data, char **attr,int len, int type) { 
   AwNode *node;
   AwString info,val;
   AwPair *pair;
   int i;
   
   info = AwString_new (len);           /* creating a new node */
   AwString_set (info,data);
   node = AwNode_new(NUM_OF_NODES,NUM_OF_NODES,len);
   AwNode_setInfo (node,info);
   AwNode_setType (node,type);
   AwString_delete (info);

   if (attr!=NULL) {
       for (i = 0; attr[i]; i += 2) {        /* set the node's attributes */
           info = AwString_new (strlen(attr[i]));
           val = AwString_new (strlen(attr[i+1]));
           AwString_set (info,(char *)attr[i]);
           AwString_set (val,(char *)attr[i+1]);
 
           pair = AwPair_new (AwString_len(info),AwString_len(val));
           AwPair_setName(pair,info);
           AwPair_setValue(pair,val);

           AwString_delete (info);
           AwString_delete (val);

           AwNode_putAttribute (node,pair);
       }
   }

   return node;    			
}



static void textElement (void *userData,const XML_Char *s,int len) {
    AwTree *tree = userData;
    AwNode *node;
    char *data;
    int i;

     data = (char *) AW_CALLOC (len+1,sizeof (char));
     strncpy (data,s,len);
     data[len]='\0';

     if (strcmp(data,"\n")==0 )                   	/* ignore lines like "\n" */
           return;

     node = makeNode (data,NULL,len,TEXT_ELEMENT); 	/* create the new node */
     
     AwTree_putNewNode(tree,node);               	/* put the new node in the tree */

     
     for (i = 0; i < AwNode_getLevel(node); i++) 	/* show it */
            putchar('\t');
     puts (AwNode_getInfo(node));  

     AW_FREE (data);
         
}



static void startElement(void *userData, const char *name, const char **attr) {
    AwTree *tree = userData;
    AwNode *node;
    AwPair *pair;
    int i;

    node = makeNode ((char *)name, (char **)attr, strlen(name), XML_ELEMENT);

    AwTree_putNewNode(tree,node);                      /* put the new node in the tree */

    
    for (i = 0; i < AwNode_getLevel(node); i++)        /* show it */
        putchar('\t');
    puts (AwNode_getInfo(node));
    AwNode_resetAttributes(node);
    pair = AwNode_nextAttribute(node);
    while (pair!=NULL) {
        for (i = 0; i < AwNode_getLevel(node) + 1 ; i++)    
            putchar('\t');
        printf ("%s = %s \n",AwPair_getName(pair),AwPair_getValue(pair));
        pair = AwNode_nextAttribute(node);
    }   
	      	 
}


static void endElement(void *userData, const char *name) {
    AwTree *tree = userData;
    AwNode *last = NULL;

    last = AwTree_getLast(tree);
    if (AwNode_getType(last)==TEXT_ELEMENT)     	/* with text element, we need to get up twice */
        AwTree_up(tree);                             	/* to reach the last xml element */

     AwTree_up(tree);
	  
}
	


size_t readBuffer (char *buf, size_t len, FILE *file)
{
    size_t readed;
    readed = fread(buf, 1, len, file);
    return readed;
}


void freenode (AwNode *n) {
    AwNode *child=NULL;
    AwPair *atr=NULL;
    int i;

    AwNode_resetAttributes(n);
    atr = AwNode_nextAttribute(n);
    while (atr!=NULL) {
	putchar ('.');    
        AwPair_delete (atr);
	atr = AwNode_nextAttribute(n);
    }

    AwNode_resetChildren (n);
    child = AwNode_nextChild(n);
    while (child!=NULL) {
	    freenode(child);
	    child = AwNode_nextChild(n);
    }

    AwNode_delete(n);     
    putchar ('.');    
}



void showTree (AwNode *n) {
    AwNode *child;
    AwPair *atr;
    int i;

    if (AwNode_getType(n)==XML_ELEMENT) 
        printf ("<%s",AwNode_getInfo(n));
    else 
        printf ("%s",AwNode_getInfo(n));
    
    AwNode_resetAttributes(n);
    atr = AwNode_nextAttribute(n);
    while (atr!=NULL) {
        printf (" %s=\'%s\'",AwPair_getName(atr),AwPair_getValue(atr));
	atr = AwNode_nextAttribute(n);
    }
    
    if (AwNode_getType(n)==XML_ELEMENT) 
        printf (">");
    
    AwNode_resetChildren (n);
    child = AwNode_nextChild(n);
    while (child!=NULL) {
        showTree(child);
	child = AwNode_nextChild(n);
    }

    if (AwNode_getType(n)==XML_ELEMENT) {
        printf ("</%s>",AwNode_getInfo(n));
    }
	
}



int main (int argc, char **argv) {
   char buf[BUFSIZ];
   int done;   
   AwTree *tree = NULL;
   XML_Parser parser;
   
   tree = AwTree_new ();

   parser = XML_ParserCreate(NULL);
   XML_SetUserData(parser, tree);
   XML_SetElementHandler(parser, startElement, endElement);
   XML_SetCharacterDataHandler(parser,textElement);

   do {
       size_t len = readBuffer (buf,sizeof(buf),stdin);
       done = len < sizeof(buf);

       if (!XML_Parse(parser, buf, len, 0 )) {
             fprintf(stderr, "%s at line %d\n", \
                              XML_ErrorString(XML_GetErrorCode(parser)), \
                              XML_GetCurrentLineNumber(parser));
             return 1;
        }

   } while (!done);

   
   printf ("Tree's info : \n");
   printf ("\troot %s maxdepth %d\n",(AwTree_getRoot(tree)==NULL)?"NULL": \
                                    AwNode_getInfo(AwTree_getRoot(tree)), \
				    AwTree_getMaxDepth(tree));

   printf ("\tlast %s depth %d\n",(AwTree_getLast(tree)==NULL)?"NULL": \
                                    AwNode_getInfo(AwTree_getLast(tree)), \
                                    AwTree_getDepth(tree));
  
   showTree(AwTree_getRoot(tree));
   putchar ('\n');
   
   freenode(AwTree_getRoot(tree));
   AwTree_delete(tree);
   XML_ParserFree(parser);

    putchar ('\n');
    return 0;
  
}
