/* -----------------------------------
** $Id: list1.c,v 1.2 2001/10/24 15:42:07 kirschpi Exp kirschpi $
** $Author: kirschpi $
** $Log: list1.c,v $
** Revision 1.2  2001/10/24 15:42:07  kirschpi
** First successful version
**
**  -------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include <malloc.h>

#define LIST_OK 1
#define LIST_FAILED -1

typedef struct _list {
    void **elements;
    int index;
    int last;
    int size;
} List;


List * List_new (int lsize) {
    List* nlist=NULL;
    
    nlist = (List *) calloc (1,sizeof (List));
    if (nlist != NULL) {
        nlist->elements = (void *) calloc (lsize,sizeof(void *));
	if (nlist->elements == NULL) {
	    free (nlist);
	    return NULL;
	}
	else {
            nlist->size = lsize;
	    nlist->index=0;
	    nlist->last=0;
	}
    }
    return nlist;
}

int List_delete (List *nlist) {
	
    if (nlist!= NULL) {
        free (nlist->elements);
	free (nlist);
	return LIST_OK;
    }	    
    return LIST_FAILED;
}	

int List_put (List *nlist, void *data) {
    List *dlist = NULL;
    
    if (nlist != NULL) {
	if (nlist->index < nlist->size) 
            nlist->elements[nlist->index++] = data;
	else {
	    /* duplicate */
	    dlist = List_duplicate (nlist);
	    if (dlist==NULL)
	        return LIST_FAILED;
	    else {
	        *nlist = *dlist;
                List_put (nlist,data);
	    } 
		
	}	
	return LIST_OK;
    }	      
    return LIST_FAILED;
}

void * List_next (List *nlist) {
    if (nlist!=NULL) {
        if (nlist->last < nlist->size) 
		return nlist->elements[nlist->last++];
    }
    return NULL; 
}

void List_reset (List *nlist) {
    if (nlist!=NULL) {
        nlist->last = 0;
    }	
}


List * List_duplicate ( List * nlist ) {
    List *dlist = NULL;
    int i;
    
    /* creates a new list */
    if (nlist != NULL) {
        dlist = List_new (nlist->size*2);
	/* copy all elements and informations to the new list */
	if (dlist != NULL) {
            dlist->last = nlist->last;
	    dlist->index = nlist->index;
	    for (i=0; i<nlist->index;i++)
		    dlist->elements[i] = nlist->elements[i];
	}
    }
    return dlist;
}

int main (int argc, char **argv) {
    int i;
    int ret;
    List *nlist = NULL;
    char *data;
    
    nlist = List_new (5);
    if (nlist== NULL) {
        fprintf (stderr, "Not enough memory\n");
        exit (1);
    }
  
    for (i=1; i<argc; i++) {
        fprintf (stdout, "Puting %s (size: %d) : ",argv[i],nlist->size);
        ret=List_put (nlist,(void *)argv[i]);	
	if (ret<0) 
            fprintf (stdout, "Error on putting element %s\n",argv[i]);	
	else
	    fprintf (stdout, "OK\n");			
    }
    
    do {
        data = (char *) List_next (nlist);
        if (data!=NULL) 
	    fprintf (stdout, "Element %s (size: %d)\n",data,nlist->size);	
    } while (data != NULL);

    return 0;
}
