#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "awtree.h"

AwNode * readnode (AwNode *pai);
AwNode * makenode (char *s); 
void addattr (AwNode *n, char *s); 
void read (char *s, int size);
void freenode (AwNode *n);
void downtree (void); 

AwTree *tree;

int main (int argc, char **argv) {
	AwNode *n=NULL;
	char s[12];

	printf ("\nCreating a tree ... ");
	tree = AwTree_new ();
	if (tree!=NULL)
		printf ("OK \n");
	else {
		printf ("FAILED \n");
		exit (0);
	}	
	
	do {
		printf ("\nNode? (y|n) ");
		read (s,128);
		s[1]='\0';
		if (strcmp(s,"n")!=0) {
			n = readnode(AwTree_getRoot(tree));
			if (AwTree_getRoot(tree)==NULL) {
				printf ("Setting root node ");
				if (AwTree_setRoot(tree,n)<0)
					printf ("FAILED \n");
				else 
					printf ("OK \n");		
			}
		/*	else {
				AwNode_putChild (AwTree_getRoot(tree),n);
			}
		*/	
		}
	} while (s[0]!='n');

	printf ("Tree's info : \n");
	printf ("\troot %s maxdepth %d\n",(AwTree_getRoot(tree)==NULL)?"NULL": \
						AwNode_getInfo(AwTree_getRoot(tree)),
						AwTree_getMaxDepth(tree));

	printf ("\tlast %s depth %d\n",(AwTree_getLast(tree)==NULL)?"NULL": \
						AwNode_getInfo(AwTree_getLast(tree)),
						AwTree_getDepth(tree));

	downtree();
	freenode (AwTree_getRoot(tree));
	printf ("Freeing tree ");
	if (AwTree_delete (tree)<0)
		printf (" FAILED \n");
	else
		printf (" OK\n");
		
	
	return 0;	
}

void downtree (void) {
	AwNode *n;
	int flag=1,done=1,count=0;
	
	printf ("setting last to root ");
	if(AwTree_setLast(tree,AwTree_getRoot(tree))<0)
		printf (" FAILED \n");
	else
		printf (" OK\n");

	n = AwTree_getRoot(tree);
	do {
		flag = 1;
		count=0;
		while (flag>0) {
			count++;
			printf ("going down ");
			flag = AwTree_down(tree);
			if (flag<0)
				printf (" FAILED \n");
			else
				printf (" OK\n");
		}	

		
		//to no fundo
		n = AwTree_getLast(tree);
		
		if (count==1 && n==AwTree_getRoot(tree))
			done = 0;
		
		if (n!=NULL) {
			printf ("Node %s level %d / depth %d (father %s) \n", \
				AwNode_getInfo(n),AwNode_getLevel(n),\
				AwTree_getDepth(tree), \
				(AwNode_getFather(n)==NULL)?"NULL":AwNode_getInfo(AwNode_getFather(n)));		
		}
		else
			printf ("FAILED\n");

		printf ("going up ");
		if (AwTree_up(tree)<0)
			printf (" FAILED \n");
		else
			printf (" OK\n");
				
		n = AwTree_getLast(tree);
		
	} while (done && n!=NULL);
}


void freenode (AwNode *n) {
	AwNode *child=NULL;
	AwPair *atr=NULL;
	
	printf ("Freeing node %s (father %s / level %d) \n",AwNode_getInfo(n), \
			   (AwNode_getFather(n)==NULL)?"NULL": \
			    AwNode_getInfo(AwNode_getFather(n)), \
	      		    AwNode_getLevel(n) );
	
	AwNode_resetAttributes (n);
	atr = AwNode_nextAttribute (n);
	while (atr!=NULL) {
		printf ("\tAttribute %s=%s ",AwPair_getName(atr),AwPair_getValue(atr));
		if (AwPair_delete(atr)<0)
			printf (" - FREE FAILED \n");
		else
			printf (" - FREE OK\n");
		atr = AwNode_nextAttribute (n);
	}

	AwNode_resetChildren (n);
	child = AwNode_nextChild(n);
	while (child!=NULL) {
		freenode(child);
		child = AwNode_nextChild(n);
	}
		
}

AwNode * readnode (AwNode *pai) {
	AwNode *n,*ptr;
	AwPair *atr;
	AwNode *child;
	char s[128];
	int i;
	
	printf ("\nNode info: ");
	read (s,128);
	
	n = makenode (s);
	printf ("Node %s \n",AwNode_getInfo(n));

	
	printf ("\nNo. of attributes: ");
	read (s,128);
	i = atoi (s);
	while (i>0) {
		printf ("\nNode attribute (name value): ");
		read (s,128);
		addattr (n,s);
		i--;
	}	
		
	AwNode_resetAttributes (n);
	atr = AwNode_nextAttribute (n);
	while (atr!=NULL) {
		printf ("\tnode attribute %s=%s",AwPair_getName(atr),AwPair_getValue(atr));
		atr = AwNode_nextAttribute(n);
	}
	
	printf ("\nPutting in the tree..");
        if (AwTree_putNewNode (tree,n) < 0)
		printf (" FAILED \n");
	else { 
		printf (" - ");
		ptr = AwTree_getLast (tree);
		if (ptr==NULL) 
			printf ("FAILED \n");			
		else {
			printf ("OK (%s father %s)\n", AwNode_getInfo(ptr), \
							(AwNode_getFather(ptr)==NULL)?"NULL": \
							AwNode_getInfo(AwNode_getFather(ptr)));
		}
		
	}
		
	//le numero de filhos
	printf ("No. of children : ");
	//fgets(s,128,stdin);
	read (s,128);
	i = atoi (s);
	while (i>0) {
		child = readnode (n);
		i--;
		
		printf ("Getting up in the tree ");		
		if (AwTree_up (tree)<0)
			printf ("FAILED \n");
		else
			printf ("OK\n");
	}

	
	return n ;		
			
}


AwNode * makenode (char *s) {
	AwNode *n;
	AwString i;

	i = AwString_new (strlen(s));
	AwString_set (i,s);
	printf ("getting info %s\n",i);
	
	n = AwNode_new (10,10,AwString_len(i));
	AwNode_setInfo (n,i);
	AwString_delete (i);
	return n;
}

void addattr (AwNode *n, char *s) {
	AwString nm,vl;
	AwPair *p;
	char *pnm,*pvl;

	pnm = s;
	pvl = strchr (s,' ');
	pvl[0]='\0';
	pvl++;
	
	printf ("name %s value %s\n",pnm,pvl);
	
	nm = AwString_new (strlen(pnm));
	vl = AwString_new (strlen(pvl));
	AwString_set (nm,pnm);
	AwString_set (vl,pvl);
	
	p = AwPair_new (AwString_len(nm),AwString_len(vl));
	AwPair_setName(p,nm);
	AwPair_setValue(p,vl);
	AwNode_putAttribute (n,p);
	
	AwString_delete (nm);
	AwString_delete (vl);
}

void read (char *s, int size) {
	int l=0;
	fgets(s,size,stdin);
	l= strlen(s);
	//printf ("readed %d last #%c#%c# - ",l,s[l-1],s[l]);		
	s[l-1]='\0';
	//printf (" #%s#\n",s);
}
